@@ -198,7 +198,7 @@ handle_exit_timeout(void *arg)
 
 	ctx = arg;
 	logerrx("timed out");
-	if (!(ctx->options & DHCPCD_MANAGER)) {
+	if (ctx->options & DHCPCD_ONESHOT) {
 		struct interface *ifp;
 
 		TAILQ_FOREACH(ifp, ctx->ifaces, next) {
@@ -208,7 +208,8 @@ handle_exit_timeout(void *arg)
 		eloop_exit(ctx->eloop, EXIT_FAILURE);
 		return;
 	}
-	ctx->options |= DHCPCD_NOWAITIP;
+
+	ctx->options |= DHCPCD_BACKGROUND;
 	dhcpcd_daemonise(ctx);
 }
 
@@ -366,7 +367,7 @@ dhcpcd_daemonise(struct dhcpcd_ctx *ctx)
 	int exit_code;
 
 	if (ctx->options & DHCPCD_DAEMONISE &&
-	    !(ctx->options & (DHCPCD_DAEMONISED | DHCPCD_NOWAITIP)))
+	    !(ctx->options & (DHCPCD_DAEMONISED | DHCPCD_BACKGROUND)))
 	{
 		if (!dhcpcd_ipwaited(ctx))
 			return;
@@ -392,7 +393,7 @@ dhcpcd_daemonise(struct dhcpcd_ctx *ctx)
 
 	eloop_event_delete(ctx->eloop, ctx->fork_fd);
 	exit_code = EXIT_SUCCESS;
-	if (write(ctx->fork_fd, &exit_code, sizeof(exit_code)) == -1)
+	if (send(ctx->fork_fd, &exit_code, sizeof(exit_code), MSG_EOR) == -1)
 		logerr(__func__);
 	close(ctx->fork_fd);
 	ctx->fork_fd = -1;
@@ -437,27 +438,62 @@ dhcpcd_drop(struct interface *ifp, int stop)
 	dhcpcd_drop_af(ifp, stop, AF_UNSPEC);
 }
 
-static void
-stop_interface(struct interface *ifp, const char *reason)
+static bool
+dhcpcd_ifrunning(struct interface *ifp)
 {
-	struct dhcpcd_ctx *ctx;
 
-	ctx = ifp->ctx;
-	loginfox("%s: removing interface", ifp->name);
-	ifp->options->options |= DHCPCD_STOPPING;
+#ifdef INET
+	if (D_CSTATE(ifp) != NULL)
+		return true;
+#ifdef IPV4LL
+	if (IPV4LL_CSTATE(ifp) != NULL)
+		return true;
+#endif
+#endif
+#ifdef DHCP6
+	if (D6_CSTATE(ifp) != NULL)
+		return true;
+#endif
+	return false;
+}
 
-	dhcpcd_drop(ifp, 1);
-	script_runreason(ifp, reason == NULL ? "STOPPED" : reason);
+void
+dhcpcd_dropped(struct interface *ifp)
+{
+	struct dhcpcd_ctx *ctx = ifp->ctx;
 
-	/* Delete all timeouts for the interfaces */
-	eloop_q_timeout_delete(ctx->eloop, ELOOP_QUEUE_ALL, NULL, ifp);
+	if (ifp->options == NULL ||
+	    !(ifp->options->options & DHCPCD_STOPPING) ||
+	    dhcpcd_ifrunning(ifp))
+		return;
 
 	/* De-activate the interface */
-	ifp->active = IF_INACTIVE;
-	ifp->options->options &= ~DHCPCD_STOPPING;
+	if (ifp->active) {
+		ifp->active = IF_INACTIVE;
+		ifp->options->options &= ~DHCPCD_STOPPING;
+		script_runreason(ifp, "STOPPED");
+	}
 
-	if (!(ctx->options & (DHCPCD_MANAGER | DHCPCD_TEST)))
-		eloop_exit(ctx->eloop, EXIT_FAILURE);
+	if (!(ctx->options & DHCPCD_EXITING))
+		return;
+
+	TAILQ_FOREACH(ifp, ctx->ifaces, next) {
+		if (dhcpcd_ifrunning(ifp))
+			break;
+	}
+
+	/* All interfaces have stopped, we can exit */
+	if (ifp == NULL)
+		eloop_exit(ctx->eloop, EXIT_SUCCESS);
+}
+
+static void
+stop_interface(struct interface *ifp)
+{
+
+	loginfox("%s: removing interface", ifp->name);
+	ifp->options->options |= DHCPCD_STOPPING;
+	dhcpcd_drop(ifp, 1);
 }
 
 static void
@@ -743,6 +779,17 @@ dhcpcd_handlecarrier(struct interface *ifp, int carrier, unsigned int flags)
 	ifp->carrier = carrier;
 	ifp->flags = flags;
 
+	/*
+	 * Inactive interfaces may not have options, we so check the
+	 * global context if we are stopping or not.
+	 * This allows an active interface to stop even if dhcpcd is not.
+	 */
+	if (ifp->options != NULL) {
+		if (ifp->options->options & DHCPCD_STOPPING)
+			return;
+	} else if (ifp->ctx->options & DHCPCD_EXITING)
+		return;
+
 	if (!if_is_link_up(ifp)) {
 		if (!ifp->active || (!was_link_up && !was_roaming))
 			return;
@@ -1069,13 +1116,16 @@ dhcpcd_handleinterface(void *arg, int action, const char *ifname)
 		}
 		if (ifp->active) {
 			logdebugx("%s: interface departed", ifp->name);
-			stop_interface(ifp, "DEPARTED");
+			stop_interface(ifp);
 		}
 		TAILQ_REMOVE(ctx->ifaces, ifp, next);
 		if_free(ifp);
 		return 0;
 	}
 
+	if (ctx->options & DHCPCD_EXITING)
+		return 0;
+
 	ifs = if_discover(ctx, &ifaddrs, -1, UNCONST(argv));
 	if (ifs == NULL) {
 		logerr(__func__);
@@ -1145,7 +1195,7 @@ dhcpcd_handlelink(void *arg, unsigned short events)
 			dhcpcd_linkoverflow(ctx);
 			return;
 		}
-		if (errno != ENOTSUP)
+		if (errno != ENOTSUP && errno != ENXIO)
 			logerr(__func__);
 	}
 }
@@ -1316,10 +1366,12 @@ if_reboot(struct interface *ifp, int argc, char **argv)
 	script_runreason(ifp, "RECONFIGURE");
 	dhcpcd_initstate1(ifp, argc, argv, 0);
 #ifdef INET
-	dhcp_reboot_newopts(ifp, oldopts);
+	if (ifp->options->options & DHCPCD_DHCP)
+		dhcp_reboot_newopts(ifp, oldopts);
 #endif
 #ifdef DHCP6
-	dhcp6_reboot(ifp);
+	if (ifp->options->options & DHCPCD_DHCP6)
+		dhcp6_reboot(ifp);
 #endif
 	dhcpcd_prestartinterface(ifp);
 }
@@ -1375,14 +1427,18 @@ reconf_reboot(struct dhcpcd_ctx *ctx, int action, int argc, char **argv, int oi)
 	}
 }
 
-static void
+static bool
 stop_all_interfaces(struct dhcpcd_ctx *ctx, unsigned long long opts)
 {
 	struct interface *ifp;
+	bool anystopped = false;
 
-	ctx->options |= DHCPCD_EXITING;
+	ctx->options |= opts;
 	if (ctx->ifaces == NULL)
-		return;
+		return anystopped;
+
+	if (ctx->options & DHCPCD_RELEASE)
+		ctx->options &= ~DHCPCD_PERSISTENT;
 
 	/* Drop the last interface first */
 	TAILQ_FOREACH_REVERSE(ifp, ctx->ifaces, if_head, next) {
@@ -1392,8 +1448,10 @@ stop_all_interfaces(struct dhcpcd_ctx *ctx, unsigned long long opts)
 		if (ifp->options->options & DHCPCD_RELEASE)
 			ifp->options->options &= ~DHCPCD_PERSISTENT;
 		ifp->options->options |= DHCPCD_EXITING;
-		stop_interface(ifp, NULL);
+		anystopped = true;
+		stop_interface(ifp);
 	}
+	return anystopped;
 }
 
 static void
@@ -1431,8 +1489,7 @@ dhcpcd_renew(struct dhcpcd_ctx *ctx)
 
 #ifdef USE_SIGNALS
 #define sigmsg "received %s, %s"
-static volatile bool dhcpcd_exiting = false;
-void
+static void
 dhcpcd_signal_cb(int sig, void *arg)
 {
 	struct dhcpcd_ctx *ctx = arg;
@@ -1444,10 +1501,11 @@ dhcpcd_signal_cb(int sig, void *arg)
 		return;
 	}
 
-	if (sig != SIGCHLD && ctx->options & DHCPCD_FORKED) {
-		if (sig != SIGHUP &&
-		    write(ctx->fork_fd, &sig, sizeof(sig)) == -1)
-			logerr("%s: write", __func__);
+	if (sig != SIGCHLD && ctx->options & DHCPCD_FORKED)
+	{
+		if (ctx->fork_fd != -1 && sig != SIGHUP &&
+		    send(ctx->fork_fd, &sig, sizeof(sig), MSG_EOR) == -1)
+			logerr("%s: send", __func__);
 		return;
 	}
 
@@ -1510,14 +1568,18 @@ dhcpcd_signal_cb(int sig, void *arg)
 	 * During teardown we don't want to process SIGTERM or SIGINT again,
 	 * as that could trigger memory issues.
 	 */
-	if (dhcpcd_exiting)
+	if (ctx->options & DHCPCD_EXITING)
 		return;
 
-	dhcpcd_exiting = true;
-	if (!(ctx->options & DHCPCD_TEST))
-		stop_all_interfaces(ctx, opts);
+	ctx->options |= DHCPCD_EXITING;
+	if (!(ctx->options & DHCPCD_TEST) &&
+	    stop_all_interfaces(ctx, opts))
+	{
+		/* We stopped something, we will exit once that is done. */
+		return;
+	}
+
 	eloop_exit(ctx->eloop, exit_code);
-	dhcpcd_exiting = false;
 }
 #endif
 
@@ -1657,8 +1719,11 @@ dumperr:
 
 	if (opts & (DHCPCD_EXITING | DHCPCD_RELEASE)) {
 		if (oifind == argc && af == AF_UNSPEC) {
-			stop_all_interfaces(ctx, opts);
-			eloop_exit(ctx->eloop, EXIT_SUCCESS);
+			ctx->options |= DHCPCD_EXITING;
+			if (stop_all_interfaces(ctx, opts) == false)
+				eloop_exit(ctx->eloop, EXIT_SUCCESS);
+			/* We did stop an interface, it will notify us once
+			 * dropped so we can exit. */
 			return 0;
 		}
 
@@ -1688,7 +1753,7 @@ dumperr:
 			if (af != AF_UNSPEC)
 				dhcpcd_drop_af(ifp, 1, af);
 			else
-				stop_interface(ifp, NULL);
+				stop_interface(ifp);
 			ifo->options = orig_opts;
 		}
 		return 0;
@@ -1890,17 +1955,28 @@ dhcpcd_pidfile_timeout(void *arg)
 	pid_t pid;
 
 	pid = pidfile_read(ctx->pidfile);
-
-	if(pid == -1)
+	if (pid == -1)
 		eloop_exit(ctx->eloop, EXIT_SUCCESS);
-	else if (++ctx->duid_len >= 100) { /* overload duid_len */
-		logerrx("pid %d failed to exit", pid);
-		eloop_exit(ctx->eloop, EXIT_FAILURE);
-	} else
+	else
 		eloop_timeout_add_msec(ctx->eloop, 100,
 		    dhcpcd_pidfile_timeout, ctx);
 }
 
+static void
+dhcpcd_exit_timeout(void *arg)
+{
+	struct dhcpcd_ctx *ctx = arg;
+	pid_t pid;
+
+	pid = pidfile_read(ctx->pidfile);
+	if (pid == -1)
+		eloop_exit(ctx->eloop, EXIT_SUCCESS);
+	else {
+		logwarnx("pid %lld failed to exit", (long long)pid);
+		eloop_exit(ctx->eloop, EXIT_FAILURE);
+	}
+}
+
 static int dup_null(int fd)
 {
 	int fd_null = open(_PATH_DEVNULL, O_WRONLY);
@@ -2234,10 +2310,14 @@ printpidfile:
 		signal(dhcpcd_signals_ignore[si], SIG_IGN);
 
 	/* Save signal mask, block and redirect signals to our handler */
-	eloop_signal_set_cb(ctx.eloop,
+	if (eloop_signal_set_cb(ctx.eloop,
 	    dhcpcd_signals, dhcpcd_signals_len,
-	    dhcpcd_signal_cb, &ctx);
-	if (eloop_signal_mask(ctx.eloop, &ctx.sigset) == -1) {
+	    dhcpcd_signal_cb, &ctx) == -1)
+	{
+		logerr("%s: eloop_signal_set_cb", __func__);
+		goto exit_failure;
+	}
+	if (eloop_signal_mask(ctx.eloop) == -1) {
 		logerr("%s: eloop_signal_mask", __func__);
 		goto exit_failure;
 	}
@@ -2245,7 +2325,7 @@ printpidfile:
 	if (sig != 0) {
 		pid = pidfile_read(ctx.pidfile);
 		if (pid != 0 && pid != -1)
-			loginfox("sending signal %s to pid %d", siga, pid);
+			loginfox("sending signal %s to pid %d", siga, (int)pid);
 		if (pid == 0 || pid == -1 || kill(pid, sig) != 0) {
 			if (pid != 0 && pid != -1 && errno != ESRCH) {
 				logerr("kill");
@@ -2258,8 +2338,10 @@ printpidfile:
 			if (sig == SIGHUP || sig == SIGUSR1)
 				goto exit_success;
 			/* Spin until it exits */
-			loginfox("waiting for pid %d to exit", pid);
+			loginfox("waiting for pid %d to exit", (int)pid);
 			dhcpcd_pidfile_timeout(&ctx);
+			eloop_timeout_add_sec(ctx.eloop, 50,
+				dhcpcd_exit_timeout, &ctx);
 			goto run_loop;
 		}
 	}
@@ -2384,7 +2466,7 @@ printpidfile:
 			else
 				logerrx(PACKAGE
 				    " already running on pid %d (%s)",
-				    pid, ctx.pidfile);
+				    (int)pid, ctx.pidfile);
 			goto exit_failure;
 		}
 	}
@@ -2402,6 +2484,7 @@ printpidfile:
 		logerr("socketpair");
 		goto exit_failure;
 	}
+
 	switch (pid = fork()) {
 	case -1:
 		logerr("fork");
@@ -2415,23 +2498,33 @@ printpidfile:
 			goto exit_failure;
 		}
 #endif
-		if (eloop_event_add(ctx.eloop, ctx.fork_fd, ELE_READ,
-		    dhcpcd_fork_cb, &ctx) == -1)
-			logerr("%s: eloop_event_add", __func__);
-
 		if (setsid() == -1) {
 			logerr("%s: setsid", __func__);
 			goto exit_failure;
 		}
 		/* Ensure we can never get a controlling terminal */
-		switch (pid = fork()) {
-		case -1:
+		pid = fork();
+		if (pid == -1) {
 			logerr("fork");
 			goto exit_failure;
-		case 0:
-			eloop_forked(ctx.eloop);
-			break;
-		default:
+		}
+		/* setsid again to ensure our child processes have the
+		 * correct ppid */
+		if (pid == 0 && setsid() == -1) {
+			logerr("%s: setsid", __func__);
+			goto exit_failure;
+		}
+		if (eloop_forked(ctx.eloop, ELF_KEEP_ALL) == -1) {
+			logerr("%s: eloop_forked", __func__);
+			goto exit_failure;
+		}
+		if (eloop_event_add(ctx.eloop, ctx.fork_fd, ELE_READ,
+		    dhcpcd_fork_cb, &ctx) == -1)
+		{
+			logerr("%s: eloop_event_add", __func__);
+			goto exit_failure;
+		}
+		if (pid != 0) {
 			ctx.options |= DHCPCD_FORKED; /* A lie */
 			i = EXIT_SUCCESS;
 			goto exit1;
@@ -2465,12 +2558,12 @@ printpidfile:
 
 	/* We have now forked, setsid, forked once more.
 	 * From this point on, we are the controlling daemon. */
-	logdebugx("spawned manager process on PID %d", getpid());
+	logdebugx("spawned manager process on PID %d", (int)getpid());
 
 start_manager:
 	ctx.options |= DHCPCD_STARTED;
 	if ((pid = pidfile_lock(ctx.pidfile)) != 0) {
-		logerr("%s: pidfile_lock %d", __func__, pid);
+		logerr("%s: pidfile_lock %d", __func__, (int)pid);
 #ifdef PRIVSEP
 		/* privsep has not started ... */
 		ctx.options &= ~DHCPCD_PRIVSEP;
@@ -2645,7 +2738,7 @@ start_manager:
 	}
 
 run_loop:
-	i = eloop_start(ctx.eloop, &ctx.sigset);
+	i = eloop_start(ctx.eloop);
 	if (i < 0) {
 		logerr("%s: eloop_start", __func__);
 		goto exit_failure;
@@ -2703,14 +2796,13 @@ exit1:
 #ifdef PRIVSEP
 	if (ps_root_stop(&ctx) == -1)
 		i = EXIT_FAILURE;
-	eloop_free(ctx.ps_eloop);
 #endif
 
 #ifdef USE_SIGNALS
 	/* If still attached, detach from the launcher */
 	if (ctx.options & DHCPCD_STARTED && ctx.fork_fd != -1) {
-		if (write(ctx.fork_fd, &i, sizeof(i)) == -1)
-			logerr("%s: write", __func__);
+		if (send(ctx.fork_fd, &i, sizeof(i), MSG_EOR) == -1)
+			logerr("%s: send", __func__);
 	}
 #endif
 
