@@ -448,10 +448,11 @@ ipv4ll_drop(struct interface *ifp)
 
 	assert(ifp != NULL);
 
+	eloop_timeout_delete(ifp->ctx->eloop, NULL, ifp);
 	ipv4ll_freearp(ifp);
 
 	if ((ifp->options->options & DHCPCD_NODROP) == DHCPCD_NODROP)
-		return;
+		goto free;
 
 	state = IPV4LL_STATE(ifp);
 	if (state) {
@@ -481,6 +482,10 @@ ipv4ll_drop(struct interface *ifp)
 		rt_build(ifp->ctx, AF_INET);
 		script_runreason(ifp, "IPV4LL");
 	}
+
+free:
+	ipv4ll_free(ifp);
+	dhcpcd_dropped(ifp);
 }
 
 void
@@ -548,7 +553,7 @@ ipv4ll_handleifa(int cmd, struct ipv4_addr *ia, pid_t pid)
 	    IN_ARE_ADDR_EQUAL(&state->addr->addr, &ia->addr))
 	{
 		loginfox("%s: pid %d deleted IP address %s",
-		    ifp->name, pid, ia->saddr);
+		    ifp->name, (int)pid, ia->saddr);
 		ipv4ll_defend_failed(ifp);
 		return ia;
 	}
