@@ -76,10 +76,10 @@ __printflike(2, 3) void logerrmessage(int pri, const char *fmt, ...);
 #define logerr(...)	log_err(__VA_ARGS__)
 #define logerrx(...)	log_errx(__VA_ARGS__)
 
-/* For logging in a chroot */
+/* For logging in a chroot using SOCK_SEQPACKET */
 int loggetfd(void);
 void logsetfd(int);
-int logreadfd(int);
+ssize_t logreadfd(int);
 
 unsigned int loggetopts(void);
 void logsetopts(unsigned int);
