@@ -71,82 +71,34 @@ struct psr_ctx {
 	struct psr_error psr_error;
 	size_t psr_datalen;
 	void *psr_data;
+	size_t psr_mdatalen;
+	void *psr_mdata;
+	bool psr_usemdata;
 };
 
-static void
-ps_root_readerrorcb(void *arg, unsigned short events)
+static ssize_t
+ps_root_readerrorcb(struct psr_ctx *psr_ctx)
 {
-	struct psr_ctx *psr_ctx = arg;
 	struct dhcpcd_ctx *ctx = psr_ctx->psr_ctx;
-	struct psr_error *psr_error = &psr_ctx->psr_error;
-	struct iovec iov[] = {
-		{ .iov_base = psr_error, .iov_len = sizeof(*psr_error) },
-		{ .iov_base = psr_ctx->psr_data,
-		  .iov_len = psr_ctx->psr_datalen },
-	};
-	ssize_t len;
-	int exit_code = EXIT_FAILURE;
-
-	if (events != ELE_READ)
-		logerrx("%s: unexpected event 0x%04x", __func__, events);
-
-#define PSR_ERROR(e)				\
-	do {					\
-		psr_error->psr_result = -1;	\
-		psr_error->psr_errno = (e);	\
-		goto out;			\
-	} while (0 /* CONSTCOND */)
-
-	len = readv(PS_ROOT_FD(ctx), iov, __arraycount(iov));
-	if (len == -1)
-		PSR_ERROR(errno);
-	else if ((size_t)len < sizeof(*psr_error))
-		PSR_ERROR(EINVAL);
-	exit_code = EXIT_SUCCESS;
-
-out:
-	eloop_exit(ctx->ps_eloop, exit_code);
-}
-
-ssize_t
-ps_root_readerror(struct dhcpcd_ctx *ctx, void *data, size_t len)
-{
-	struct psr_ctx psr_ctx = {
-	    .psr_ctx = ctx,
-	    .psr_data = data, .psr_datalen = len,
-	};
 	int fd = PS_ROOT_FD(ctx);
-
-	if (eloop_event_add(ctx->ps_eloop, fd, ELE_READ,
-	    ps_root_readerrorcb, &psr_ctx) == -1)
-		return -1;
-
-	eloop_enter(ctx->ps_eloop);
-	eloop_start(ctx->ps_eloop, &ctx->sigset);
-	eloop_event_delete(ctx->ps_eloop, fd);
-
-	errno = psr_ctx.psr_error.psr_errno;
-	return psr_ctx.psr_error.psr_result;
-}
-
-#ifdef PRIVSEP_GETIFADDRS
-static void
-ps_root_mreaderrorcb(void *arg, unsigned short events)
-{
-	struct psr_ctx *psr_ctx = arg;
-	struct dhcpcd_ctx *ctx = psr_ctx->psr_ctx;
 	struct psr_error *psr_error = &psr_ctx->psr_error;
 	struct iovec iov[] = {
 		{ .iov_base = psr_error, .iov_len = sizeof(*psr_error) },
 		{ .iov_base = NULL, .iov_len = 0 },
 	};
 	ssize_t len;
-	int exit_code = EXIT_FAILURE;
 
-	if (events != ELE_READ)
-		logerrx("%s: unexpected event 0x%04x", __func__, events);
+#define PSR_ERROR(e)				\
+	do {					\
+		psr_error->psr_result = -1;	\
+		psr_error->psr_errno = (e);	\
+		return -1;			\
+	} while (0 /* CONSTCOND */)
 
-	len = recv(PS_ROOT_FD(ctx), psr_error, sizeof(*psr_error), MSG_PEEK);
+	if (eloop_waitfd(fd) == -1)
+		PSR_ERROR(errno);
+
+	len = recv(fd, psr_error, sizeof(*psr_error), MSG_PEEK);
 	if (len == -1)
 		PSR_ERROR(errno);
 	else if ((size_t)len < sizeof(*psr_error))
@@ -154,48 +106,74 @@ ps_root_mreaderrorcb(void *arg, unsigned short events)
 
 	if (psr_error->psr_datalen > SSIZE_MAX)
 		PSR_ERROR(ENOBUFS);
-	else if (psr_error->psr_datalen != 0) {
-		psr_ctx->psr_data = malloc(psr_error->psr_datalen);
-		if (psr_ctx->psr_data == NULL)
+	if (psr_ctx->psr_usemdata &&
+	    psr_error->psr_datalen > psr_ctx->psr_mdatalen)
+	{
+		void *d = realloc(psr_ctx->psr_mdata, psr_error->psr_datalen);
+		if (d == NULL)
 			PSR_ERROR(errno);
-		psr_ctx->psr_datalen = psr_error->psr_datalen;
-		iov[1].iov_base = psr_ctx->psr_data;
-		iov[1].iov_len = psr_ctx->psr_datalen;
+		psr_ctx->psr_mdata = d;
+		psr_ctx->psr_mdatalen = psr_error->psr_datalen;
+	}
+	if (psr_error->psr_datalen != 0) {
+		if (psr_ctx->psr_usemdata)
+			iov[1].iov_base = psr_ctx->psr_mdata;
+		else {
+			if (psr_error->psr_datalen > psr_ctx->psr_datalen)
+				PSR_ERROR(ENOBUFS);
+			iov[1].iov_base = psr_ctx->psr_data;
+		}
+		iov[1].iov_len = psr_error->psr_datalen;
 	}
 
-	len = readv(PS_ROOT_FD(ctx), iov, __arraycount(iov));
+	len = readv(fd, iov, __arraycount(iov));
 	if (len == -1)
 		PSR_ERROR(errno);
-	else if ((size_t)len != sizeof(*psr_error) + psr_ctx->psr_datalen)
+	else if ((size_t)len != sizeof(*psr_error) + psr_error->psr_datalen)
 		PSR_ERROR(EINVAL);
-	exit_code = EXIT_SUCCESS;
+	return len;
+}
 
-out:
-	eloop_exit(ctx->ps_eloop, exit_code);
+ssize_t
+ps_root_readerror(struct dhcpcd_ctx *ctx, void *data, size_t len)
+{
+	struct psr_ctx *pc = ctx->ps_root->psp_data;
+
+	pc->psr_data = data;
+	pc->psr_datalen = len;
+	pc->psr_usemdata = false;
+	ps_root_readerrorcb(pc);
+
+	errno = pc->psr_error.psr_errno;
+	return pc->psr_error.psr_result;
 }
 
 ssize_t
 ps_root_mreaderror(struct dhcpcd_ctx *ctx, void **data, size_t *len)
 {
-	struct psr_ctx psr_ctx = {
-	    .psr_ctx = ctx,
-	};
-	int fd = PS_ROOT_FD(ctx);
+	struct psr_ctx *pc = ctx->ps_root->psp_data;
+	void *d;
 
-	if (eloop_event_add(ctx->ps_eloop, fd, ELE_READ,
-	    ps_root_mreaderrorcb, &psr_ctx) == -1)
-		return -1;
+	pc->psr_usemdata = true;
+	ps_root_readerrorcb(pc);
 
-	eloop_enter(ctx->ps_eloop);
-	eloop_start(ctx->ps_eloop, &ctx->sigset);
-	eloop_event_delete(ctx->ps_eloop, fd);
+	if (pc->psr_error.psr_datalen != 0) {
+		if (pc->psr_error.psr_datalen > pc->psr_mdatalen) {
+			errno = EINVAL;
+			return -1;
+		}
+		d = malloc(pc->psr_error.psr_datalen);
+		if (d == NULL)
+			return -1;
+		memcpy(d, pc->psr_mdata, pc->psr_error.psr_datalen);
+	} else
+		d = NULL;
 
-	errno = psr_ctx.psr_error.psr_errno;
-	*data = psr_ctx.psr_data;
-	*len = psr_ctx.psr_datalen;
-	return psr_ctx.psr_error.psr_result;
+	errno = pc->psr_error.psr_errno;
+	*data = d;
+	*len = pc->psr_error.psr_datalen;
+	return pc->psr_error.psr_result;
 }
-#endif
 
 static ssize_t
 ps_root_writeerror(struct dhcpcd_ctx *ctx, ssize_t result,
@@ -210,6 +188,7 @@ ps_root_writeerror(struct dhcpcd_ctx *ctx, ssize_t result,
 		{ .iov_base = &psr, .iov_len = sizeof(psr) },
 		{ .iov_base = data, .iov_len = len },
 	};
+	struct msghdr msg = { .msg_iov = iov, .msg_iovlen = __arraycount(iov) };
 	ssize_t err;
 	int fd = PS_ROOT_FD(ctx);
 
@@ -217,17 +196,17 @@ ps_root_writeerror(struct dhcpcd_ctx *ctx, ssize_t result,
 	logdebugx("%s: result %zd errno %d", __func__, result, errno);
 #endif
 
-	err = writev(fd, iov, __arraycount(iov));
+	err = sendmsg(fd, &msg, MSG_EOR);
 
 	/* Error sending the message? Try sending the error of sending. */
-	if (err == -1) {
+	if (err == -1 && errno != EPIPE) {
 		logerr("%s: result=%zd, data=%p, len=%zu",
 		    __func__, result, data, len);
 		psr.psr_result = err;
 		psr.psr_errno = errno;
 		iov[1].iov_base = NULL;
 		iov[1].iov_len = 0;
-		err = writev(fd, iov, __arraycount(iov));
+		err = sendmsg(fd, &msg, MSG_EOR);
 	}
 
 	return err;
@@ -784,7 +763,7 @@ ps_root_signalcb(int sig, void *arg)
 	if (!(ctx->options & DHCPCD_EXITING))
 		return;
 	if (!(ps_waitforprocs(ctx)))
-		eloop_exit(ctx->ps_eloop, EXIT_SUCCESS);
+		eloop_exit(ctx->eloop, EXIT_SUCCESS);
 }
 
 int (*handle_interface)(void *, int, const char *);
@@ -864,6 +843,17 @@ ps_root_log(void *arg, unsigned short events)
 		logerr(__func__);
 }
 
+static void
+ps_root_freepsdata(void *arg)
+{
+	struct psr_ctx *pc = arg;
+
+	if (pc == NULL)
+		return;
+	free(pc->psr_mdata);
+	free(pc);
+}
+
 pid_t
 ps_root_start(struct dhcpcd_ctx *ctx)
 {
@@ -872,8 +862,9 @@ ps_root_start(struct dhcpcd_ctx *ctx)
 		.psi_cmd = PS_ROOT,
 	};
 	struct ps_process *psp;
-	int logfd[2], datafd[2];
+	int logfd[2] = { -1, -1}, datafd[2] = { -1, -1};
 	pid_t pid;
+	struct psr_ctx *pc;
 
 	if (xsocketpair(AF_UNIX, SOCK_SEQPACKET | SOCK_CXNB, 0, logfd) == -1)
 		return -1;
@@ -884,6 +875,7 @@ ps_root_start(struct dhcpcd_ctx *ctx)
 
 	if (xsocketpair(AF_UNIX, SOCK_SEQPACKET | SOCK_CXNB, 0, datafd) == -1)
 		return -1;
+
 	if (ps_setbuf_fdpair(datafd) == -1)
 		return -1;
 #ifdef PRIVSEP_RIGHTS
@@ -891,10 +883,27 @@ ps_root_start(struct dhcpcd_ctx *ctx)
 		return -1;
 #endif
 
+	pc = calloc(1, sizeof(*pc));
+	if (pc == NULL)
+		return -1;
+	pc->psr_ctx = ctx;
+
 	psp = ctx->ps_root = ps_newprocess(ctx, &id);
+	if (psp == NULL)
+	{
+		free(pc);
+		return -1;
+	}
+	psp->psp_freedata = ps_root_freepsdata;
 	strlcpy(psp->psp_name, "privileged proxy", sizeof(psp->psp_name));
 	pid = ps_startprocess(psp, ps_root_recvmsg, NULL,
-	    ps_root_startcb, ps_root_signalcb, PSF_ELOOP);
+	    ps_root_startcb, PSF_ELOOP);
+	if (pid == -1) {
+		free(pc);
+		return -1;
+	}
+
+	psp->psp_data = pc;
 
 	if (pid == 0) {
 		ctx->ps_log_fd = logfd[0]; /* Keep open to pass to processes */
@@ -915,7 +924,7 @@ ps_root_start(struct dhcpcd_ctx *ctx)
 	close(datafd[1]);
 	if (eloop_event_add(ctx->eloop, ctx->ps_data_fd, ELE_READ,
 	    ps_root_dispatch, ctx) == -1)
-		return 1;
+		return -1;
 
 	return pid;
 }
@@ -950,17 +959,37 @@ int
 ps_root_stop(struct dhcpcd_ctx *ctx)
 {
 	struct ps_process *psp = ctx->ps_root;
+	int err;
 
-	if (!(ctx->options & DHCPCD_PRIVSEP) ||
-	    ctx->eloop == NULL)
+	if (!(ctx->options & DHCPCD_PRIVSEP))
 		return 0;
 
 	/* If we are the root process then remove the pidfile */
-	if (ctx->options & DHCPCD_PRIVSEPROOT &&
-	    !(ctx->options & DHCPCD_TEST))
-	{
-		if (unlink(ctx->pidfile) == -1)
+	if (ctx->options & DHCPCD_PRIVSEPROOT) {
+		if (!(ctx->options & DHCPCD_TEST) && unlink(ctx->pidfile) == -1)
 			logerr("%s: unlink: %s", __func__, ctx->pidfile);
+
+		/* drain the log */
+		if (ctx->ps_log_root_fd != -1) {
+			ssize_t loglen;
+
+#ifdef __linux__
+			/* Seems to help to get the last parts,
+			 * sched_yield(2) does not. */
+			sleep(0);
+#endif
+			do {
+				loglen = logreadfd(ctx->ps_log_root_fd);
+			} while (loglen != 0 && loglen != -1);
+			close(ctx->ps_log_root_fd);
+			ctx->ps_log_root_fd = -1;
+		}
+	}
+
+	if (ctx->ps_data_fd != -1) {
+		eloop_event_delete(ctx->eloop, ctx->ps_data_fd);
+		close(ctx->ps_data_fd);
+		ctx->ps_data_fd = -1;
 	}
 
 	/* Only the manager process gets past this point. */
@@ -981,7 +1010,10 @@ ps_root_stop(struct dhcpcd_ctx *ctx)
 			return -1;
 	} /* else the root process has already exited :( */
 
-	return ps_stopwait(ctx);
+	err = ps_stopwait(ctx);
+	if (ctx->ps_root != NULL)
+		ps_freeprocess(ctx->ps_root);
+	return err;
 }
 
 ssize_t
